#"A Nonparametric Statistical Approach to Content Analysis of Items"
#Stats

#Diego Marcondes and Nilton Rogerio Marcondes
#Instituto de Matemática e Estatística, Universidade de São Paulo, Brasil
#Tel.: +55 11 97585-6644
#dmarcondes@ime.usp.br

#####Chochran's test#####

cochranQ <- function(m,ci){
  #GET THE MODE FOR EACH ITEM
  getmode <- function(v){
    uniqv <- unique(v)
    uniqv[which.max(tabulate(match(v, uniqv)))]
  }
  n <- floor(ncol(m) * ci)
  
  #EXCLUDE THE ITEMS BY THE CONCORDANCE INDEX
  tmp <- apply(X = m,MARGIN = 1,FUN = getmode)
  tmp <- cbind(m,tmp)
  number <- apply(X = tmp,MARGIN = 1,FUN = function(x) sum(x[-length(x)] == x[length(x)]))
  m <- tmp[number > n,]
  if(is.null(dim(m)))
    return(list("Q" = NA,"pvalue" = NA,"items" = NA))
  if(dim(m)[1] == 0)
    return(list("Q" = NA,"pvalue" = NA,"items" = NA))
  t <- matrix(nrow = nrow(m),ncol = ncol(m)-1)
  rownames(t) <- rownames(m)
  colnames(t) <- colnames(m)[-ncol(m)]
  
  #BUILDING THE TABLE FOR THE Q TEST
  for(i in 1:nrow(m))
    t[i,] <- as.numeric(m[i,-ncol(m)] == m[i,ncol(m)])
  R <- rowSums(t)
  D <- colSums(t)
  
  #TEST STATISTIC
  s <- ncol(t)
  N <- sum(D)
  d <- sum(R * (s - R))
  Q <- s * (s-1) * sum((D - N/s)^2)/d
  
  #PVALUE
  pvalue <- pchisq(q = Q,df = s - 1,lower.tail = FALSE)
  
  #RETURN
  return(list("Q" = Q,"pvalue" = pvalue,"t" = t,"items" = nrow(t)))
}
